package com.ekexiu.portal.service;

import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;

import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.DefaultValue;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Delete;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.apt.web.annotation.param.PathVar;
import org.jfw.util.PageQueryResult;
import org.jfw.util.StringUtil;

import com.ekexiu.portal.dao.ImageDao;
import com.ekexiu.portal.dao.OperationDao;
import com.ekexiu.portal.dao.ProfessorDao;
import com.ekexiu.portal.dao.ResourceDao;
import com.ekexiu.portal.dao.WatchDao;
import com.ekexiu.portal.po.Operation;
import com.ekexiu.portal.po.Resource;
import com.ekexiu.portal.pojo.EditProfessor;

@Path("/resource")
public class ResourceService {
	@Autowrie
	private ProfessorDao professorDao;
	@Autowrie
	private WatchDao watchDao;
	@Autowrie
	private ImageDao imageDao;
	@Autowrie
	private ResourceDao resourceDao;
	@Autowrie
	private ImagesService imagesService;
	@Autowrie
	private ImageService imageService;
	@Autowrie
	private OperationDao operationDao;

	public ProfessorDao getProfessorDao() {
		return professorDao;
	}

	public void setProfessorDao(ProfessorDao professorDao) {
		this.professorDao = professorDao;
	}

	public WatchDao getWatchDao() {
		return watchDao;
	}

	public void setWatchDao(WatchDao watchDao) {
		this.watchDao = watchDao;
	}

	public OperationDao getOperationDao() {
		return operationDao;
	}

	public void setOperationDao(OperationDao operationDao) {
		this.operationDao = operationDao;
	}

	public ImageDao getImageDao() {
		return imageDao;
	}

	public void setImageDao(ImageDao imageDao) {
		this.imageDao = imageDao;
	}

	public ImageService getImageService() {
		return imageService;
	}

	public void setImageService(ImageService imageService) {
		this.imageService = imageService;
	}

	public ResourceDao getResourceDao() {
		return resourceDao;
	}

	public void setResourceDao(ResourceDao resourceDao) {
		this.resourceDao = resourceDao;
	}

	public ImagesService getImagesService() {
		return imagesService;
	}

	public void setImagesService(ImagesService imagesService) {
		this.imagesService = imagesService;
	}

	@Post
	@Path
	public String insert(@JdbcConn(true) Connection con, Resource resource, @Nullable String fn) {
		String resourceId = StringUtil.buildUUID();
		try {
			if(null != fn){
				this.imagesService.insert(con, resourceId, fn);
			}
			resource.setResourceId(resourceId);
			this.resourceDao.insert(con, resource);
		} catch (SQLException | IOException e) {
			try {
				con.rollback();
			} catch (SQLException e1) {
				e1.printStackTrace();
			}
			e.printStackTrace();
		}
		return resourceId;
	}
	
	@Post
	@Path("/saveRes")
	public String insertRes(@JdbcConn(true) Connection con, Resource resource, @Nullable String base64, Integer angle) {
		String resourceId = StringUtil.buildUUID();
		try {
			if(null != base64){
				this.imagesService.insertResourceImg(con, resourceId, base64, angle);
			}
			resource.setResourceId(resourceId);
			this.resourceDao.insert(con, resource);
		} catch (SQLException | IOException e) {
			try {
				con.rollback();
			} catch (SQLException e1) {
				e1.printStackTrace();
			}
			e.printStackTrace();
		}
		return resourceId;
	}

	@Post
	@Path("/updateResource")
	public void update(@JdbcConn(true) Connection con, Resource resource, @Nullable String fn) throws SQLException, IOException {
		if(null != fn){
			this.imagesService.insert(con, resource.getResourceId(), fn);
		}
		this.resourceDao.update(con, resource);
	}

	@Get
	@Path("/{resourceId}")
	public Resource query(@JdbcConn Connection con, @PathVar String resourceId) throws SQLException {
		return this.resourceDao.query(con, resourceId);
	}
	
	@Get
	@Path("/resourceInfo")
	public Resource queryInfo(@JdbcConn Connection con, String resourceId) throws SQLException {
		Resource resource = this.resourceDao.queryInfo(con, resourceId);
		EditProfessor professor = this.professorDao.queryBaseInfo(con, resource.getProfessorId());
		if(professor != null){
			professor.setHasHeadImage(this.imageService.hasProfessorImage(professor.getId()));
		}
		resource.setProfessor(professor);
		return resource;
	}
	
	@Get
	@Path("/pqall")
	public PageQueryResult<Resource> queryPageAll(@JdbcConn Connection con, @DefaultValue("10") int pageSize, @DefaultValue("1") int pageNo) throws SQLException {
		return this.resourceDao.queryPageAll(con, pageSize, pageNo);
	}
	
	@Get
	@Path("/qapro")
	public List<Resource> queryPro(@JdbcConn Connection con, String professorId) throws SQLException {
		return this.resourceDao.queryPro(con, professorId);
	}
	
	@Post
	@Path("/nameAndSupport")
	public void updateNameAndSupport(@JdbcConn(true) Connection con, String resourceId, 
			String resourceName, String supportedServices) throws SQLException{
		this.resourceDao.updateNameAndSupport(con, resourceId, resourceName, supportedServices);
	}
	
	@Post
	@Path("/subject")
	public void updateSubject(@JdbcConn(true) Connection con,String resourceId,@Nullable String subject) throws SQLException {
		this.resourceDao.updateSubject(con, resourceId, subject);
	}

	@Post
	@Path("/industry")
	public void updateIndustry(@JdbcConn(true) Connection con,String resourceId,@Nullable String industry) throws SQLException {
		this.resourceDao.updateIndustry(con, resourceId, industry);
	}

	@Post
	@Path("/descp")
	public void updateDescp(@JdbcConn(true) Connection con,String resourceId,@Nullable String descp) throws SQLException {
		this.resourceDao.updateDescp(con, resourceId, descp);
	}
	
	@Post
	@Path("/hopePayMethod")
	public void updateHopePayMethod(@JdbcConn(true) Connection con,String resourceId,@Nullable String hopePayMethod) throws SQLException {
		this.resourceDao.updateHopePayMethod(con, resourceId, hopePayMethod);
	}
	
	@Post
	@Path("/cooperationNotes")
	public void updateCooperationNotes(@JdbcConn(true) Connection con,String resourceId,@Nullable String cooperationNotes) throws SQLException{
		this.resourceDao.updateCooperationNotes(con, resourceId, cooperationNotes);
	}
	
	@Delete
	@Path("/{resourceId}")
	public boolean delete(@JdbcConn(true) Connection con, @PathVar String resourceId) throws SQLException {
		List<Operation> operations = this.operationDao.queryResourceId(con, resourceId);
		if(operations.isEmpty()){
			this.imageDao.deleteRes(con, resourceId);
			this.resourceDao.delete(con, resourceId);
			this.watchDao.deleteWatch(con, resourceId);
			return true;
		}else{
			return false;
		}
	}
	
	@Get
	@Path("/pqRes")
	public PageQueryResult<Resource> queryPageRes(@JdbcConn(false) Connection con, @Nullable String key, 
			@Nullable String subject, @Nullable String industry, @Nullable String province, @Nullable String address, 
			@Nullable Integer authType, @DefaultValue("10") int pageSize, @DefaultValue("1") int pageNo) throws SQLException {
		if(key!=null) key="%"+key+"%";
		if(subject!=null) subject ="%"+subject+"%";
		if(industry!=null)industry="%"+industry+"%";
		if(province!=null)province="%"+province+"%";
		if(address!=null)address="%"+address+"%";
		PageQueryResult<Resource> queryResult =  this.resourceDao.queryPageRes(con, key, subject, industry, province, address, authType, pageSize, pageNo);
		List<Resource> resources = queryResult.getData();
		if(resources != null){
			for (Resource resource : resources) {
				resource.setProfessor(this.professorDao.queryBaseInfo(con, resource.getProfessorId()));
				resource.setImages(this.imageDao.queryRes(con, resource.getResourceId()));
			}
		}
		return queryResult;
	}
	
	@Get
	@Path("/pq")
	public PageQueryResult<Resource> queryPage(@JdbcConn(false) Connection con, @Nullable String key, 
			@Nullable String subject, @Nullable String industry, @Nullable String address, 
			@DefaultValue("10") int pageSize, @DefaultValue("1") int pageNo) throws SQLException {
		if(key!=null) key="%"+key+"%";
		if(subject!=null) subject ="%"+subject+"%";
		if(industry!=null)industry="%"+industry+"%";
		if(address!=null)address="%"+address+"%";
		PageQueryResult<Resource> queryResult = this.resourceDao.queryPage(con, key, subject, industry, address, pageSize, pageNo);
		List<Resource> resources = queryResult.getData();
		if(resources != null){
			for (Resource resource : resources) {
				resource.setImages(this.imageDao.queryRes(con, resource.getResourceId()));
			}
		}
		return queryResult;
	}
	
}
